@@ -1,7 +1,6 @@
 from __future__ import unicode_literals
 
 import logging
-import random
 import urlparse
 
 from mopidy.audio import PlaybackState
@@ -9,7 +8,7 @@ from mopidy.audio import PlaybackState
 from . import listener
 
 
-logger = logging.getLogger('mopidy.core')
+logger = logging.getLogger(__name__)
 
 
 class PlaybackController(object):
@@ -21,35 +20,18 @@ class PlaybackController(object):
         self.core = core
 
         self._state = PlaybackState.STOPPED
-        self._shuffled = []
-        self._first_shuffle = True
         self._volume = None
+        self._mute = False
 
     def _get_backend(self):
         if self.current_tl_track is None:
             return None
         uri = self.current_tl_track.track.uri
         uri_scheme = urlparse.urlparse(uri).scheme
-        return self.backends.with_playback_by_uri_scheme.get(uri_scheme, None)
+        return self.backends.with_playback.get(uri_scheme, None)
 
     ### Properties
 
-    def get_consume(self):
-        return getattr(self, '_consume', False)
-
-    def set_consume(self, value):
-        if self.get_consume() != value:
-            self._trigger_options_changed()
-        return setattr(self, '_consume', value)
-
-    consume = property(get_consume, set_consume)
-    """
-    :class:`True`
-        Tracks are removed from the playlist when they have been played.
-    :class:`False`
-        Tracks are not removed from the playlist.
-    """
-
     def get_current_tl_track(self):
         return self.current_tl_track
 
@@ -69,56 +51,6 @@ class PlaybackController(object):
     Read-only. Extracted from :attr:`current_tl_track` for convenience.
     """
 
-    def get_random(self):
-        return getattr(self, '_random', False)
-
-    def set_random(self, value):
-        if self.get_random() != value:
-            self._trigger_options_changed()
-        return setattr(self, '_random', value)
-
-    random = property(get_random, set_random)
-    """
-    :class:`True`
-        Tracks are selected at random from the playlist.
-    :class:`False`
-        Tracks are played in the order of the playlist.
-    """
-
-    def get_repeat(self):
-        return getattr(self, '_repeat', False)
-
-    def set_repeat(self, value):
-        if self.get_repeat() != value:
-            self._trigger_options_changed()
-        return setattr(self, '_repeat', value)
-
-    repeat = property(get_repeat, set_repeat)
-    """
-    :class:`True`
-        The current playlist is played repeatedly. To repeat a single track,
-        select both :attr:`repeat` and :attr:`single`.
-    :class:`False`
-        The current playlist is played once.
-    """
-
-    def get_single(self):
-        return getattr(self, '_single', False)
-
-    def set_single(self, value):
-        if self.get_single() != value:
-            self._trigger_options_changed()
-        return setattr(self, '_single', value)
-
-    single = property(get_single, set_single)
-    """
-    :class:`True`
-        Playback is stopped after current song, unless in :attr:`repeat`
-        mode.
-    :class:`False`
-        Playback continues after current song.
-    """
-
     def get_state(self):
         return self._state
 
@@ -156,119 +88,6 @@ class PlaybackController(object):
     time_position = property(get_time_position)
     """Time position in milliseconds."""
 
-    def get_tracklist_position(self):
-        if self.current_tl_track is None:
-            return None
-        try:
-            return self.core.tracklist.tl_tracks.index(self.current_tl_track)
-        except ValueError:
-            return None
-
-    tracklist_position = property(get_tracklist_position)
-    """
-    The position of the current track in the tracklist.
-
-    Read-only.
-    """
-
-    def get_tl_track_at_eot(self):
-        tl_tracks = self.core.tracklist.tl_tracks
-
-        if not tl_tracks:
-            return None
-
-        if self.random and not self._shuffled:
-            if self.repeat or self._first_shuffle:
-                logger.debug('Shuffling tracks')
-                self._shuffled = tl_tracks
-                random.shuffle(self._shuffled)
-                self._first_shuffle = False
-
-        if self.random and self._shuffled:
-            return self._shuffled[0]
-
-        if self.current_tl_track is None:
-            return tl_tracks[0]
-
-        if self.repeat and self.single:
-            return tl_tracks[self.tracklist_position]
-
-        if self.repeat and not self.single:
-            return tl_tracks[(self.tracklist_position + 1) % len(tl_tracks)]
-
-        try:
-            return tl_tracks[self.tracklist_position + 1]
-        except IndexError:
-            return None
-
-    tl_track_at_eot = property(get_tl_track_at_eot)
-    """
-    The track that will be played at the end of the current track.
-
-    Read-only. A :class:`mopidy.models.TlTrack`.
-
-    Not necessarily the same track as :attr:`tl_track_at_next`.
-    """
-
-    def get_tl_track_at_next(self):
-        tl_tracks = self.core.tracklist.tl_tracks
-
-        if not tl_tracks:
-            return None
-
-        if self.random and not self._shuffled:
-            if self.repeat or self._first_shuffle:
-                logger.debug('Shuffling tracks')
-                self._shuffled = tl_tracks
-                random.shuffle(self._shuffled)
-                self._first_shuffle = False
-
-        if self.random and self._shuffled:
-            return self._shuffled[0]
-
-        if self.current_tl_track is None:
-            return tl_tracks[0]
-
-        if self.repeat:
-            return tl_tracks[(self.tracklist_position + 1) % len(tl_tracks)]
-
-        try:
-            return tl_tracks[self.tracklist_position + 1]
-        except IndexError:
-            return None
-
-    tl_track_at_next = property(get_tl_track_at_next)
-    """
-    The track that will be played if calling :meth:`next()`.
-
-    Read-only. A :class:`mopidy.models.TlTrack`.
-
-    For normal playback this is the next track in the playlist. If repeat
-    is enabled the next track can loop around the playlist. When random is
-    enabled this should be a random track, all tracks should be played once
-    before the list repeats.
-    """
-
-    def get_tl_track_at_previous(self):
-        if self.repeat or self.consume or self.random:
-            return self.current_tl_track
-
-        if self.tracklist_position in (None, 0):
-            return None
-
-        return self.core.tracklist.tl_tracks[self.tracklist_position - 1]
-
-    tl_track_at_previous = property(get_tl_track_at_previous)
-    """
-    The track that will be played if calling :meth:`previous()`.
-
-    A :class:`mopidy.models.TlTrack`.
-
-    For normal playback this is the previous track in the playlist. If
-    random and/or consume is enabled it should return the current track
-    instead.
-    """
-
     def get_volume(self):
         if self.audio:
             return self.audio.get_volume().get()
@@ -288,6 +107,26 @@ class PlaybackController(object):
     volume = property(get_volume, set_volume)
     """Volume as int in range [0..100] or :class:`None`"""
 
+    def get_mute(self):
+        if self.audio:
+            return self.audio.get_mute().get()
+        else:
+            # For testing
+            return self._mute
+
+    def set_mute(self, value):
+        value = bool(value)
+        if self.audio:
+            self.audio.set_mute(value)
+        else:
+            # For testing
+            self._mute = value
+
+        self._trigger_mute_changed(value)
+
+    mute = property(get_mute, set_mute)
+    """Mute state as a :class:`True` if muted, :class:`False` otherwise"""
+
     ### Methods
 
     def change_track(self, tl_track, on_error_step=1):
@@ -318,15 +157,14 @@ class PlaybackController(object):
             return
 
         original_tl_track = self.current_tl_track
+        next_tl_track = self.core.tracklist.eot_track(original_tl_track)
 
-        if self.tl_track_at_eot:
-            self._trigger_track_playback_ended()
-            self.play(self.tl_track_at_eot)
+        if next_tl_track:
+            self.change_track(next_tl_track)
         else:
             self.stop(clear_current_track=True)
 
-        if self.consume:
-            self.core.tracklist.remove(tlid=original_tl_track.tlid)
+        self.core.tracklist.mark_played(original_tl_track)
 
     def on_tracklist_change(self):
         """
@@ -334,12 +172,7 @@ class PlaybackController(object):
 
         Used by :class:`mopidy.core.TracklistController`.
         """
-        self._first_shuffle = True
-        self._shuffled = []
-
-        if (not self.core.tracklist.tl_tracks or
-                self.current_tl_track not in
-                self.core.tracklist.tl_tracks):
+        if self.current_tl_track not in self.core.tracklist.tl_tracks:
             self.stop(clear_current_track=True)
 
     def next(self):
@@ -349,9 +182,9 @@ class PlaybackController(object):
         The current playback state will be kept. If it was playing, playing
         will continue. If it was paused, it will still be paused, etc.
         """
-        if self.tl_track_at_next:
-            self._trigger_track_playback_ended()
-            self.change_track(self.tl_track_at_next)
+        tl_track = self.core.tracklist.next_track(self.current_tl_track)
+        if tl_track:
+            self.change_track(tl_track)
         else:
             self.stop(clear_current_track=True)
 
@@ -374,37 +207,43 @@ class PlaybackController(object):
         :type on_error_step: int, -1 or 1
         """
 
-        if tl_track is not None:
-            assert tl_track in self.core.tracklist.tl_tracks
-        elif tl_track is None:
+        assert on_error_step in (-1, 1)
+
+        if tl_track is None:
             if self.state == PlaybackState.PAUSED:
                 return self.resume()
-            elif self.current_tl_track is not None:
-                tl_track = self.current_tl_track
-            elif self.current_tl_track is None and on_error_step == 1:
-                tl_track = self.tl_track_at_next
-            elif self.current_tl_track is None and on_error_step == -1:
-                tl_track = self.tl_track_at_previous
 
-        if tl_track is not None:
-            self.current_tl_track = tl_track
-            self.state = PlaybackState.PLAYING
-            backend = self._get_backend()
-            if not backend or not backend.playback.play(tl_track.track).get():
-                logger.warning('Track is not playable: %s', tl_track.track.uri)
-                if self.random and self._shuffled:
-                    self._shuffled.remove(tl_track)
+            if self.current_tl_track is not None:
+                tl_track = self.current_tl_track
+            else:
                 if on_error_step == 1:
-                    # TODO: can cause an endless loop for single track repeat.
-                    self.next()
+                    tl_track = self.core.tracklist.next_track(tl_track)
                 elif on_error_step == -1:
-                    self.previous()
+                    tl_track = self.core.tracklist.previous_track(tl_track)
+
+            if tl_track is None:
                 return
 
-        if self.random and self.current_tl_track in self._shuffled:
-            self._shuffled.remove(self.current_tl_track)
+        assert tl_track in self.core.tracklist.tl_tracks
 
-        self._trigger_track_playback_started()
+        if self.state == PlaybackState.PLAYING:
+            self.stop()
+
+        self.current_tl_track = tl_track
+        self.state = PlaybackState.PLAYING
+        backend = self._get_backend()
+        success = backend and backend.playback.play(tl_track.track).get()
+
+        if success:
+            self.core.tracklist.mark_playing(tl_track)
+            self._trigger_track_playback_started()
+        else:
+            self.core.tracklist.mark_unplayable(tl_track)
+            if on_error_step == 1:
+                # TODO: can cause an endless loop for single track repeat.
+                self.next()
+            elif on_error_step == -1:
+                self.previous()
 
     def previous(self):
         """
@@ -413,8 +252,9 @@ class PlaybackController(object):
         The current playback state will be kept. If it was playing, playing
         will continue. If it was paused, it will still be paused, etc.
         """
-        self._trigger_track_playback_ended()
-        self.change_track(self.tl_track_at_previous, on_error_step=-1)
+        tl_track = self.current_tl_track
+        self.change_track(
+            self.core.tracklist.previous_track(tl_track), on_error_step=-1)
 
     def resume(self):
         """If paused, resume playing the current track."""
@@ -467,8 +307,8 @@ class PlaybackController(object):
         if self.state != PlaybackState.STOPPED:
             backend = self._get_backend()
             if not backend or backend.playback.stop().get():
-                self._trigger_track_playback_ended()
                 self.state = PlaybackState.STOPPED
+                self._trigger_track_playback_ended()
         if clear_current_track:
             self.current_tl_track = None
 
@@ -510,14 +350,14 @@ class PlaybackController(object):
             'playback_state_changed',
             old_state=old_state, new_state=new_state)
 
-    def _trigger_options_changed(self):
-        logger.debug('Triggering options changed event')
-        listener.CoreListener.send('options_changed')
-
     def _trigger_volume_changed(self, volume):
         logger.debug('Triggering volume changed event')
         listener.CoreListener.send('volume_changed', volume=volume)
 
+    def _trigger_mute_changed(self, mute):
+        logger.debug('Triggering mute changed event')
+        listener.CoreListener.send('mute_changed', mute=mute)
+
     def _trigger_seeked(self, time_position):
         logger.debug('Triggering seeked event')
         listener.CoreListener.send('seeked', time_position=time_position)
