@@ -69,10 +69,14 @@ class ImmutableObject(object):
         data = {}
         for key in self.__dict__.keys():
             public_key = key.lstrip('_')
-            data[public_key] = values.pop(public_key, self.__dict__[key])
+            value = values.pop(public_key, self.__dict__[key])
+            if value is not None:
+                data[public_key] = value
         for key in values.keys():
             if hasattr(self, key):
-                data[key] = values.pop(key)
+                value = values.pop(key)
+                if value is not None:
+                    data[key] = value
         if values:
             raise TypeError(
                 'copy() got an unexpected keyword argument "%s"' % key)
@@ -136,6 +140,76 @@ def model_json_decoder(dct):
     return dct
 
 
+class Ref(ImmutableObject):
+    """
+    Model to represent URI references with a human friendly name and type
+    attached. This is intended for use a lightweight object "free" of metadata
+    that can be passed around instead of using full blown models.
+
+    :param uri: object URI
+    :type uri: string
+    :param name: object name
+    :type name: string
+    :param type: object type
+    :type name: string
+    """
+
+    #: The object URI. Read-only.
+    uri = None
+
+    #: The object name. Read-only.
+    name = None
+
+    #: The object type, e.g. "artist", "album", "track", "playlist",
+    #: "directory". Read-only.
+    type = None
+
+    #: Constant used for comparison with the :attr:`type` field.
+    ALBUM = 'album'
+
+    #: Constant used for comparison with the :attr:`type` field.
+    ARTIST = 'artist'
+
+    #: Constant used for comparison with the :attr:`type` field.
+    DIRECTORY = 'directory'
+
+    #: Constant used for comparison with the :attr:`type` field.
+    PLAYLIST = 'playlist'
+
+    #: Constant used for comparison with the :attr:`type` field.
+    TRACK = 'track'
+
+    @classmethod
+    def album(cls, **kwargs):
+        """Create a :class:`Ref` with ``type`` :attr:`ALBUM`."""
+        kwargs['type'] = Ref.ALBUM
+        return cls(**kwargs)
+
+    @classmethod
+    def artist(cls, **kwargs):
+        """Create a :class:`Ref` with ``type`` :attr:`ARTIST`."""
+        kwargs['type'] = Ref.ARTIST
+        return cls(**kwargs)
+
+    @classmethod
+    def directory(cls, **kwargs):
+        """Create a :class:`Ref` with ``type`` :attr:`DIRECTORY`."""
+        kwargs['type'] = Ref.DIRECTORY
+        return cls(**kwargs)
+
+    @classmethod
+    def playlist(cls, **kwargs):
+        """Create a :class:`Ref` with ``type`` :attr:`PLAYLIST`."""
+        kwargs['type'] = Ref.PLAYLIST
+        return cls(**kwargs)
+
+    @classmethod
+    def track(cls, **kwargs):
+        """Create a :class:`Ref` with ``type`` :attr:`TRACK`."""
+        kwargs['type'] = Ref.TRACK
+        return cls(**kwargs)
+
+
 class Artist(ImmutableObject):
     """
     :param uri: artist URI
@@ -204,8 +278,8 @@ class Album(ImmutableObject):
     # actual usage of this field with more than one image.
 
     def __init__(self, *args, **kwargs):
-        self.__dict__['artists'] = frozenset(kwargs.pop('artists', []))
-        self.__dict__['images'] = frozenset(kwargs.pop('images', []))
+        self.__dict__['artists'] = frozenset(kwargs.pop('artists', None) or [])
+        self.__dict__['images'] = frozenset(kwargs.pop('images', None) or [])
         super(Album, self).__init__(*args, **kwargs)
 
 
@@ -219,6 +293,12 @@ class Track(ImmutableObject):
     :type artists: list of :class:`Artist`
     :param album: track album
     :type album: :class:`Album`
+    :param composers: track composers
+    :type composers: string
+    :param performers: track performers
+    :type performers: string
+    :param genre: track genre
+    :type genre: string
     :param track_no: track number in album
     :type track_no: integer
     :param disc_no: disc number in album
@@ -229,6 +309,8 @@ class Track(ImmutableObject):
     :type length: integer
     :param bitrate: bitrate in kbit/s
     :type bitrate: integer
+    :param comment: track comment
+    :type comment: string
     :param musicbrainz_id: MusicBrainz ID
     :type musicbrainz_id: string
     :param last_modified: Represents last modification time
@@ -247,6 +329,15 @@ class Track(ImmutableObject):
     #: The track :class:`Album`. Read-only.
     album = None
 
+    #: A set of track composers. Read-only.
+    composers = frozenset()
+
+    #: A set of track performers`. Read-only.
+    performers = frozenset()
+
+    #: The track genre. Read-only.
+    genre = None
+
     #: The track number in the album. Read-only.
     track_no = 0
 
@@ -262,6 +353,9 @@ class Track(ImmutableObject):
     #: The track's bitrate in kbit/s. Read-only.
     bitrate = None
 
+    #: The track comment. Read-only.
+    comment = None
+
     #: The MusicBrainz ID of the track. Read-only.
     musicbrainz_id = None
 
@@ -271,7 +365,10 @@ class Track(ImmutableObject):
     last_modified = 0
 
     def __init__(self, *args, **kwargs):
-        self.__dict__['artists'] = frozenset(kwargs.pop('artists', []))
+        get = lambda key: frozenset(kwargs.pop(key, None) or [])
+        self.__dict__['artists'] = get('artists')
+        self.__dict__['composers'] = get('composers')
+        self.__dict__['performers'] = get('performers')
         super(Track, self).__init__(*args, **kwargs)
 
 
@@ -340,7 +437,7 @@ class Playlist(ImmutableObject):
     last_modified = None
 
     def __init__(self, *args, **kwargs):
-        self.__dict__['tracks'] = tuple(kwargs.pop('tracks', []))
+        self.__dict__['tracks'] = tuple(kwargs.pop('tracks', None) or [])
         super(Playlist, self).__init__(*args, **kwargs)
 
     # TODO: def insert(self, pos, track): ... ?
@@ -376,7 +473,7 @@ class SearchResult(ImmutableObject):
     albums = tuple()
 
     def __init__(self, *args, **kwargs):
-        self.__dict__['tracks'] = tuple(kwargs.pop('tracks', []))
-        self.__dict__['artists'] = tuple(kwargs.pop('artists', []))
-        self.__dict__['albums'] = tuple(kwargs.pop('albums', []))
+        self.__dict__['tracks'] = tuple(kwargs.pop('tracks', None) or [])
+        self.__dict__['artists'] = tuple(kwargs.pop('artists', None) or [])
+        self.__dict__['albums'] = tuple(kwargs.pop('albums', None) or [])
         super(SearchResult, self).__init__(*args, **kwargs)
