@@ -12,7 +12,7 @@ import pykka
 from mopidy.utils import encoding
 
 
-logger = logging.getLogger('mopidy.utils.server')
+logger = logging.getLogger(__name__)
 
 
 class ShouldRetrySocketCall(Exception):
@@ -291,7 +291,7 @@ class Connection(object):
         return True
 
     def timeout_callback(self):
-        self.stop('Client timeout out after %s seconds' % self.timeout)
+        self.stop('Client inactive for %ds; closing connection' % self.timeout)
         return False
 
 
